package com.caimei.service.user.impl;

import com.caimei.entity.*;
import com.caimei.mapper.products.HomePageMapper;
import com.caimei.mapper.user.PersonalMapper;
import com.caimei.service.user.PersonalService;
import com.github.pagehelper.PageHelper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class PersonalServiceImpl implements PersonalService {
    protected static final Logger logger = LoggerFactory.getLogger(PersonalServiceImpl.class);

    @Autowired
    private PersonalMapper personalMapper;
    @Autowired
    private HomePageMapper homePageMapper;

    @Override
    public CmMallClub myCentre(CmMallOperationUser operationUser) {
        CmMallClub cmMallClub = personalMapper.myCentre(operationUser);
        if (cmMallClub != null) {
            Double ableUserMoney = cmMallClub.getAbleUserMoney();
            String userMoney = String.format("%.2f", ableUserMoney);
            cmMallClub.setUserMoney(userMoney);
            //购物车数量
            Integer count = homePageMapper.cartQuantity(cmMallClub.getUserID());
            if (count == null) count = 0;
            logger.info("------------PersonalServiceImpl.myCentre.count:" + count);
            cmMallClub.setCartCount(count);
            //待付款数量
            Integer paymentCount = personalMapper.findOrderCount(cmMallClub.getUserID(), 1, operationUser.getUserOrganizeID());
            if (paymentCount == null) paymentCount = 0;
            cmMallClub.setPaymentCount(paymentCount);
            //待发货数量
            Integer waitShipmentsCount = personalMapper.findOrderCount(cmMallClub.getUserID(), 2, operationUser.getUserOrganizeID());
            if (waitShipmentsCount == null) waitShipmentsCount = 0;
            cmMallClub.setWaitShipmentsCount(waitShipmentsCount);
            //已发货数量
            Integer shipmentsCount = personalMapper.findOrderCount(cmMallClub.getUserID(), 3, operationUser.getUserOrganizeID());
            if (shipmentsCount == null) shipmentsCount = 0;
            cmMallClub.setShipmentsCount(shipmentsCount);
            //退货款数量
            Integer salesReturnCount = personalMapper.findOrderCount(cmMallClub.getUserID(), 4, operationUser.getUserOrganizeID());
            if (salesReturnCount == null) salesReturnCount = 0;
            cmMallClub.setSalesReturnCount(salesReturnCount);
        }
        return cmMallClub;
    }

    @Override
    public Map<String, Object> touchBalance(Integer index, Integer pageSize, int year, int month, CmMallUserBalanceRecord balanceRecord) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month - 1);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getMinimum(Calendar.DATE));
        String startTime = format.format(calendar.getTime()) + " 00:00:00";
        balanceRecord.setStartTime(startTime);
        calendar.set(Calendar.DAY_OF_MONTH, calendar.getActualMaximum(Calendar.DATE));
        String endTime = format.format(calendar.getTime()) + " 23:59:59";
        balanceRecord.setEndTime(endTime);
        if (index == null) index = 1;
        if (pageSize == null) pageSize = 10;
        PageHelper.startPage(index, pageSize);
        List<CmMallUserBalanceRecord> list = personalMapper.touchBalance(balanceRecord);
        CmMallPage<CmMallUserBalanceRecord> page = new CmMallPage<>(list);
        Double ableUserMoney = personalMapper.ableUserMoney(balanceRecord.getUserId());
        if (ableUserMoney == null) ableUserMoney = 0d;
        String userMoney = String.format("%.2f", ableUserMoney);
        Map<String, Object> map = new HashMap();
        map.put("page", page);
        map.put("ableUserMoney", userMoney);
        return map;
    }

    @Override
    public List<CmMallProvince> address() {
        List<CmMallProvince> list = personalMapper.findAllProvince();
        for (CmMallProvince cmMallProvince : list) {
            List<CmMallCity> cmMallCityList = personalMapper.findAllCity(cmMallProvince.getProvinceid());
            cmMallProvince.setCitys(cmMallCityList);
            if (cmMallCityList != null && cmMallCityList.size() > 0) {
                for (CmMallCity cmMallCity : cmMallCityList) {
                    List<CmMallTown> cmMallTownList = personalMapper.findAllTown(cmMallCity.getCityid());
                    cmMallCity.setTowns(cmMallTownList);
                }
            }
        }
        return list;
    }

    @Override
    public List<CmMallAddress> findAddress(Integer userID) {
        List<CmMallAddress> cmMallAddressList = personalMapper.findAddress(userID);
        for (CmMallAddress cmMallAddress : cmMallAddressList) {
            CmMallAddress cantonal = personalMapper.findCantonal(cmMallAddress.getTownID());
            if (cantonal != null) {
                cmMallAddress.setProvince(cantonal.getProvince());
                cmMallAddress.setCity(cantonal.getCity());
                cmMallAddress.setTown(cantonal.getTown());
            }
        }
        return cmMallAddressList;
    }

    @Override
    public List<CmMallProvince> getProvince() {
        List<CmMallProvince> list = personalMapper.findAllProvince();
        return list;
    }

    @Override
    public List<CmMallCity> getCity(Long provinceid) {
        return personalMapper.findAllCity(provinceid);
    }

    @Override
    public List<CmMallTown> getTown(Integer cityid) {
        return personalMapper.findAllTown(cityid);
    }

    @Override
    public void saveAddress(CmMallAddress cmMallAddress) {
        if (cmMallAddress.getDefaultFlag() != null && cmMallAddress.getDefaultFlag().equals("1")) {
            CmMallAddress addr = personalMapper.findDefaultAddress(cmMallAddress.getUserID());
            if (addr != null) {
                addr.setDefaultFlag("0");
                personalMapper.updateAddress(addr);
            }
        }
        if (cmMallAddress.getAddressID() == null) {
            personalMapper.saveAddress(cmMallAddress);
        } else {
            personalMapper.updateAddress(cmMallAddress);
        }
    }

    @Override
    public void deleteAddress(Integer addressID) {
        personalMapper.deleteAddress(addressID);
    }

    @Override
    public CmWxJsonModel invoice(CmMallUserInvoiceInfo cmMallUserInvoiceInfo) {
        CmWxJsonModel res = CmWxJsonModel.newInstance();
        CmMallUserInvoiceInfo invoiceInfo = personalMapper.findInvoice(cmMallUserInvoiceInfo.getUserId());
        if (invoiceInfo == null && cmMallUserInvoiceInfo.getInvoiceTitle() != null) {
            personalMapper.invoice(cmMallUserInvoiceInfo);
        } else if (invoiceInfo != null && cmMallUserInvoiceInfo.getInvoiceTitle() != null) {
            personalMapper.updateInvoice(cmMallUserInvoiceInfo);
        }
        CmMallUserInvoiceInfo info = personalMapper.findInvoice(cmMallUserInvoiceInfo.getUserId());
        return res.success(info);
    }
}
