package com.caimei.utils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei.module.base.entity.bo.JsonModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * 微信工具类
 *
 * @author LG
 * @create 2017-09-21
 **/
@Component
public class WxConfig {
    protected static final Logger logger = LoggerFactory.getLogger(WxConfig.class);

    public static String MiniAppId;

    @Value("${miniprogram.AppId1}")
    public void setMiniAppId(String miniAppId) {
        MiniAppId = miniAppId;
    }

    public static String MiniAppSecret;

    @Value("${miniprogram.AppSecret1}")
    public void setMiniAppSecret(String miniAppSecret) {
        MiniAppSecret = miniAppSecret;
    }

    public static String CrmAppId;

    @Value("${miniprogram.crm_AppId}")
    public void setCrmAppId(String crmAppId) {
        CrmAppId = crmAppId;
    }

    public static String CrmAppSecret;

    @Value("${miniprogram.crm_AppSecret}")
    public void setCrmAppSecret(String crmAppSecret) {
        CrmAppSecret = crmAppSecret;
    }


    /**
     * 网页授权登录,通过code获取access_token
     *
     * @param code wxcode
     * @return
     * @throws Exception
     */
    public static Map<String, Object> getAccessTokenMap(String code, String source) throws Exception {
        String link = "https://api.weixin.qq.com/sns/oauth2/access_token?appid=APPID&secret=SECRET&code=CODE&grant_type=authorization_code";
        //微信公众号
        link = link.replace("APPID", CrmAppId);
        link = link.replace("SECRET", CrmAppSecret);
        //填写第一步获取的code参数
        link = link.replace("CODE", code);
        String result = HttpRequest.sendGet(link);
        logger.info(result);
        Map<String, Object> map = JSONObject.parseObject(result, Map.class);
        return map;
    }


    /**
     * 小程序微信授权登录,获取openid
     *
     * @param code 微信凭证
     */
    public static JsonModel getWxJscode(String code, HttpServletRequest request) {
        JsonModel res = JsonModel.newInstance();
        logger.info("Start get SessionKey");
        Map<String, Object> map = new HashMap<>();
        //获取当前微信小程序的环境
        String referer = request.getHeader("Referer");
        logger.info("referer-is----:" + referer);
        map.put("referer", referer);
        String requestUrl = "https://api.weixin.qq.com/sns/jscode2session";
        Map<String, String> requestUrlParam = new HashMap<String, String>();
        //小程序appId
        requestUrlParam.put("appid", MiniAppId);
        logger.info("appid: ---" + MiniAppId);
        //小程序appsecret
        requestUrlParam.put("secret", MiniAppSecret);
        //小程序端返回的code
        requestUrlParam.put("js_code", code);
        //默认参数
        requestUrlParam.put("grant_type", "authorization_code");
        //发送post请求读取调用微信接口获取openid用户唯一标识
        String infos;
        try {
            infos = HttpRequest.sendPost(requestUrl, requestUrlParam);
        } catch (Exception e) {
            res.setData(map);
            return res.error("服务器内部异常");
        }
        //解析相应内容（转换成json对象）
        JSONObject jsonObject = JSON.parseObject(infos);
        String openid = jsonObject.getString("openid");
        logger.info("openid----->" + openid);
        String unionid = jsonObject.getString("unionid");
        logger.info("unionid------>" + unionid);
        map.put("openid", openid);
        map.put("unionid", unionid);
        String session_key = jsonObject.getString("session_key");
        map.put("session_key", session_key);
        String errcode = jsonObject.getString("errcode");
        String errmsg = jsonObject.getString("errmsg");
        if (!StringUtils.isEmpty(errcode) &&
                ("-1".equals(errcode) || "40029".equals(errcode) || "45011".equals(errcode))) {
            res.setMsg(errmsg);
            res.setData(map);
            map.put("sessionKey", session_key);
            res.setCode(-1);
            return res;
        }
        return res.success(map);
    }
}
